#ifndef _MAPSERVER_VERSION_H
#define _MAPSERVER_VERSION_H

#define MS_VERSION_MAJOR 8
#define MS_VERSION_MINOR 0
#define MS_VERSION_REV 0
#define MS_VERSION "8.0.0"
#define MS_VERSION_NUM  80000 

#endif

